#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Do something for uninstalling the TMANS."""

import os

import ionas.request as request
from product_conf import (LIBEXEC_DIR, STOR_IF_TO_VOL, )
from fal_info import (FAL_SHARES_PATH, FAL_SERVICECTRL_PATH, ReqTypes)

import manualctrl


SHARES_DELETE_XML = """<shares><share><name>tmadmin</name></share></shares>"""


HANDLER_FOR_SERVICE = os.path.join(LIBEXEC_DIR, "evh", "evh_tmnas_service.py")
INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]

UNMOUNT_EVENT_XML = '<events><eventinfo category="volume" ' + \
                                'action="unmount" previous="true">' + \
                    '<volume><name>' + INTERNAL_VOL + '</name></volume>' + \
                    '</eventinfo></events>'

SERVICECTRL_SAMBA_REQ_XML = """<servicectrl>
    <servicelist>
            <service><name>samba</name></service>
    </servicelist>
</servicectrl>"""


def __send_request(target, type, body):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def stop_tmnas_service():
    """Stop service for the TMNAS by a event handler."""

    __send_request(HANDLER_FOR_SERVICE, ReqTypes.EVENT, UNMOUNT_EVENT_XML)


def suspend_samba_service():
    """Suspend samba service."""

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SUSPEND,
                    SERVICECTRL_SAMBA_REQ_XML)


def resume_samba_service():
    """Resume samba service."""

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.RESUME,
                    SERVICECTRL_SAMBA_REQ_XML)


def delete_tmadmin_share():
    """Delete "tmadmin" share folder."""

    __send_request(FAL_SHARES_PATH, ReqTypes.DELETE, SHARES_DELETE_XML)


#
# Main
#

suspend_samba_service()
stop_tmnas_service()
delete_tmadmin_share()
resume_samba_service()

manualctrl.uninstall()
